-- objective not resetting when dragging a new item onto bar to replace an existing one

local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:FormatAlert(itemTable, oldCount, newCount)
    local difference = newCount - oldCount
    local alert, soundID

    if oldCount ~= newCount then
        if itemTable.objective then
            alert = string.format(
                L["%s %s %s%d/%d|r (%s%d|r)"],
                (oldCount < itemTable.objective and newCount >= itemTable.objective) and L["Objective complete!"] or L["Farming update:"],
                select(2, GetItemInfo(itemTable.itemID)) or "",
                newCount >= itemTable.objective and "|cff00ff00" or "|cffffcc00",
                newCount or "",
                itemTable.objective,
                difference > 0 and "|cff00ff00+" or "|cffff0000",
                difference or ""
            )

            soundID = (oldCount < itemTable.objective and newCount >= itemTable.objective) and 619
        else
            alert = string.format(
                L["Farming update: %s x%d (%s%d|r)"],
                select(2, GetItemInfo(itemTable.itemID)) or "",
                newCount or "",
                difference > 0 and "|cff00ff00+" or "|cffff0000",
                difference or ""
            )
        end

        soundID = soundID or (oldCount < newCount and 864)
    end

    return alert, soundID
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:GetPoints(barDB, barAnchor, isButton1)
    local padding = db.settings.style == "minimal" and 1 or 0
    local points = {
        ["Down"] = {"TOP", barAnchor, "BOTTOM", 0, -(padding * (isButton1 and 1 or 2))},
        ["Left"] = {"RIGHT", barAnchor, "LEFT", -(padding * (isButton1 and 1 or 2)), 0},
        ["Right"] = {"LEFT", barAnchor, "RIGHT", padding * (isButton1 and 1 or 2), 0},
        ["Up"] = {"BOTTOM", barAnchor, "TOP", 0, padding * (isButton1 and 1 or 2)},
    }

    return points[barDB.orientation]
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

local button1, button2
function f:LoadButtons(barFrame)
    local barDB = barFrame.db
    local style = f:GetStyle(db.settings.style)
    
    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Button frames

    for i = 1, f.maxButtons do
        local button = barFrame.buttons[i] or CreateFrame("Button", "$parentbutton" .. i, barFrame)
        button:SetSize(40, 40)
        button:ClearAllPoints()
        button:SetPoint(f:unpack(f:GetPoints(barDB, i == 1 and barFrame or barFrame.buttons[i - 1], i == 1)))
        f:SetFrameTexture(button, style.button)

        button.id = i
        barFrame.buttons[i] = button

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Icon frame

        button.icon = button.icon or CreateFrame("Button", "$parent" .. i .. "Icon", button, "SecureActionButtonTemplate, SecureHandlerDragTemplate")
        button.icon:SetPoint("TOPLEFT", 2, -2)
        button.icon:SetPoint("BOTTOMRIGHT", -2, 2)
        button.icon:SetPushedTextOffset(0, 0)
        if barDB.style == "default" then
            button.icon:SetSize(button:GetWidth() - 6, button:GetHeight() - 6)
        else
            button.icon:SetSize(button:GetWidth(), button:GetHeight())
        end

        local overlay = button.icon.overlay or button.icon:CreateTexture(nil, "OVERLAY")
        button.icon.overlay = overlay
        overlay:SetTexture((barDB.items[i] and barDB.items[i].includeBank) and "Interface\\BUTTONS\\UI-AutoCastableOverlay" or "")
        overlay:SetTexCoord(.16, .81, .16, .81)
        overlay:SetSize(button:GetWidth() + 8, button:GetHeight() + 8)
        overlay:SetPoint("CENTER", 0, 0)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Icon scripts

        button.icon:SetScript("OnEvent", function(self, event, ...)
            if (event == "COMBAT_LOG_EVENT_UNFILTERED" or event == "BAG_UPDATE" or event == "BAG_UPDATE_COOLDOWN" or event == "UNIT_INVENTORY_CHANGED") and self:GetParent().itemID then
                local count = GetItemCount(GetItemInfo(self:GetParent().itemID), barDB.items[i].includeBank) or 0

                local alert, soundID = f:FormatAlert(barDB.items[i], tonumber(self:GetFontString():GetText()) or 0, count)
                if db.settings.alerts.chat and alert then
                    f:print(alert)
                end
                if db.settings.alerts.screen and alert then
                    UIErrorsFrame:AddMessage(alert, 1, 1, 1)
                end
                if db.settings.alerts.sound and soundID then
                    PlaySound(soundID)
                end

                self:GetFontString():SetText(count > 0 and count or "")
                f:UpdateObjectiveCount(button, barDB.items[button.id])
            elseif event == "PLAYER_REGEN_ENABLED" then
                f:SetButtonID(self, self:GetParent().itemID)
                self:UnregisterEvent("PLAYER_REGEN_ENABLED")
            end
        end)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

        button.icon:RegisterForClicks("AnyUp")
        button.icon:SetScript("PreClick", function(self, mouseButton, ...)
            local cursorType, cursorID = GetCursorInfo()
            if not UnitAffectingCombat("player") and cursorID and ("item:" .. cursorID) ~= self:GetParent().itemID then
                self:SetAttribute("type", nil)
            end
        end)
        
        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

        button.icon:SetScript("PostClick", function(self, mouseButton, ...)
            local button = self:GetParent()
            local barFrame = button:GetParent()

            local cursorType, cursorID = GetCursorInfo()
            if cursorType == "item" then
                f:SetButtonID(self, cursorID)
            elseif not cursorType then
                if mouseButton == "RightButton" then
                    if IsShiftKeyDown() and button.itemID then
                        f:SetButtonID(self)
                    elseif IsControlKeyDown() then
                        button.itemIDEditBox:Show()
                        button.itemIDEditBox:SetFocus()
                        button.itemIDEditBox:SetText(button.itemID or "")
                        C_Timer.After(.01, function()
                            button.itemIDEditBox:HighlightText()
                        end)
                    else
                        if IsUsableItem(button.itemID) then
                            f:print(string.format(L["Using %s"], select(2, GetItemInfo(button.itemID))))
                        end
                    end
                elseif mouseButton == "LeftButton" then
                    if IsShiftKeyDown() and button.itemID then
                        if barDB.items[i].includeBank then
                            barDB.items[i].includeBank = false
                        else
                            barDB.items[i].includeBank = true
                        end

                        f:SetButtonID(self, button.itemID)
                        f:print(string.format(L["Bar %d button %d: Include Bank = %s"], barFrame.barID, i, barDB.items[i].includeBank and "TRUE" or "FALSE"))
                    elseif IsControlKeyDown() and button.itemID then
                        button.objectiveEditBox:Show()
                        button.objectiveEditBox:SetFocus()
                        button.objectiveEditBox:SetText(barDB.items[i] and barDB.items[i].objective or "0")
                        C_Timer.After(.01, function()
                            button.objectiveEditBox:HighlightText()
                        end)
                    elseif not UnitAffectingCombat("player") then
                        if not button1 and button.itemID then
                            button1 = {
                                icon = self,
                                itemID = button.itemID,
                                itemTable = barDB.items[i],
                            }

                            self:GetNormalTexture():SetDesaturated(true)
                        elseif button1 then
                            button2 = {
                                itemID = button.itemID,
                                itemTable = barDB.items[i],
                            }


                            f:SetButtonID(self, button1.itemID, button1.itemTable)
                            f:SetButtonID(button1.icon, button2.itemID, button2.itemTable)

                            button1.icon:GetNormalTexture():SetDesaturated()

                            button1 = nil
                            button2 = nil
                        end
                    elseif button.itemID then
                        f:print(L["Cannot move items during combat. Try again when combat ends."])
                    end
                end
            end
        end)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Tooltip

        button.icon:SetScript("OnEnter", function(self, ...)
            if db.settings.tooltip.button and self:GetParent().itemID then
                local itemID = self:GetParent().itemID
                local _, _, _, ilvl, _, itemType, subtype, stackSize = GetItemInfo(itemID)
                local count = GetItemCount(itemID, barDB.items[i].includeBank)
                GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT", 50, -100)
                GameTooltip:SetHyperlink("item:" .. itemID)
                GameTooltip:AddLine(" ")
                GameTooltip:AddLine(string.format(L["%s%s"], itemType or "", subtype == itemType and "" or string.format(" (%s)", subtype)), 1, .82, 0)
                GameTooltip:AddLine(string.format(L["Stack Size: %d"], stackSize), 1, 1, 1)
                GameTooltip:AddLine(string.format(L["Item Level: %d"], ilvl), 1, 1, 1)
                GameTooltip:AddLine(string.format(L["Item ID: %d"],itemID), 1, 1, 1)
                GameTooltip:AddLine(" ")
                GameTooltip:AddLine(string.format(L["|cff00ff00Button ID: %d|r"], i), 1, 1, 1)
                GameTooltip:AddLine(string.format(L["Count: %s %s"], count, count > 0 and string.format("(%s stacks)", f:round(count / stackSize, 1)) or ""), 1, 1, 1)
                if barDB.items[i].objective then
                    GameTooltip:AddLine(string.format(L["Objective: %s"], barDB.items[i].objective), 1, 1, 1)
                end
                GameTooltip:AddLine(string.format(L["Include Bank: %s"], barDB.items[i].includeBank and "TRUE" or "FALSE"), 1, .82, 0)
                if barDB.items[i].objective then
                    GameTooltip:AddLine(string.format(L["Objective Complete: %s"], GetItemCount(itemID, barDB.items[i].includeBank) >= barDB.items[i].objective and "TRUE" or "FALSE"), 1, .82, 0)
                end
                GameTooltip:AddLine(" ")
                GameTooltip:AddLine("------------------------------------------------", 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffClick|r to select/move an item."], 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffShift+Click|r toggles bank inclusion."], 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffCtrl+Click|r to set a farming objective."], 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffRight-Click|r to USE the item."], 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffShift+Right-Click|r clears the slot."], 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffCtrl+Right-Click|r to enter an Item ID."], 1, 1, 1)
                GameTooltip:Show()
            end
        end)

        button.icon:SetScript("OnLeave", function(self, ...)
            GameTooltip:Hide()
        end)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Font strings

        local itemCount = button.icon:CreateFontString(nil, "OVERLAY")
        itemCount:SetFont("Fonts\\ARIALN.TTF", 14, "OUTLINE")
        itemCount:SetJustifyH("RIGHT")
        itemCount:SetSize(button:GetWidth() - 4, 15)
        itemCount:SetPoint("BOTTOMRIGHT", -4, 4)
        button.icon:SetFontString(itemCount)

        local objectiveCount = button.icon:CreateFontString(nil, "OVERLAY", style.fonts.headerOutline)
        objectiveCount:SetJustifyH("LEFT")
        objectiveCount:SetPoint("TOPLEFT", 2, -2)
        button.objectiveCount = objectiveCount

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- EditBoxes

        local objectiveEditBox = button.objectiveEditBox or CreateFrame("EditBox", "$parentObjectiveEditBox", button)
        objectiveEditBox:SetSize(75, 20)
        objectiveEditBox:SetPoint("TOPLEFT", 0, 0)
        objectiveEditBox:SetAutoFocus(false)

        objectiveEditBox:SetFrameStrata("TOOLTIP")
        objectiveEditBox:SetFontObject(GameFontHighlightSmall)
        objectiveEditBox:SetTextInsets(1, 1, 1, 1)
        objectiveEditBox:SetMaxLetters(9)
        objectiveEditBox:SetJustifyH("LEFT")       

        if style.buttonEditBox1 then
            local textureFrame = CreateFrame("Frame", nil, objectiveEditBox)
            f:SetFrameTexture(objectiveEditBox, style.buttonEditBox1)
            textureFrame:SetAllPoints(objectiveEditBox)
        end
        
        objectiveEditBox:SetScript("OnEscapePressed", function(self) self:ClearFocus(); self:Hide() end)
        objectiveEditBox:SetScript("OnEditFocusLost", function(self) self:ClearFocus(); self:Hide() end)

        objectiveEditBox:SetScript("OnTextChanged", function(self)
            local text = string.gsub(self:GetText(), "[%s%c%p%a]", "")
            self:SetText(text)
        end)

        objectiveEditBox:SetScript("OnEnterPressed", function(self)
            local objective = tonumber(self:GetText())

            if objective and objective > 0 then
                button.objectiveCount:SetText(f:ParseNumber(objective))

                if GetItemCount(button.itemID,  barDB.items[i] and barDB.items[i].includeBank) >= objective then
                    button.objectiveCount:SetTextColor(0, 1, 0)
                else
                    button.objectiveCount:SetTextColor(1, .82, 0)
                end

                barDB.items[i].objective = objective
                f:print(string.format(L["Farming objective set: %d %s"], barDB.items[i].objective, select(2, GetItemInfo(button.itemID))))
                if db.settings.alerts.sound then
                    PlaySound(618)
                end
            else
                barDB.items[i].objective = false
                button.objectiveCount:SetText("")
                f:print(L["Farming objective abandoned."])
                if db.settings.alerts.sound then
                    PlaySound(846)
                end
            end

            self:ClearFocus()
            self:Hide()
        end)

        objectiveEditBox:Hide()
        button.objectiveEditBox = objectiveEditBox
        
        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 

        local itemIDEditBox = button.itemIDEditBox or CreateFrame("EditBox", "$parentItemIDEditBox", button)
        itemIDEditBox:SetSize(50, 20)
        itemIDEditBox:SetPoint("TOP", button, "BOTTOM", 0, -4)
        itemIDEditBox:SetAutoFocus(false)

        itemIDEditBox:SetFrameStrata("TOOLTIP")
        itemIDEditBox:SetFontObject(GameFontHighlightSmall)
        itemIDEditBox:SetTextInsets(1, 1, 1, 1)
        itemIDEditBox:SetMaxLetters(7)
        itemIDEditBox:SetJustifyH("CENTER")

        if style.buttonEditBox2 then
            local textureFrame = CreateFrame("Frame", nil, itemIDEditBox)
            f:SetFrameTexture(itemIDEditBox, style.buttonEditBox2)
            textureFrame:SetAllPoints(itemIDEditBox)
        end
        
        itemIDEditBox:SetScript("OnEscapePressed", function(self) self:ClearFocus(); self:Hide() end)
        itemIDEditBox:SetScript("OnEditFocusLost", function(self) self:ClearFocus(); self:Hide() end)

        itemIDEditBox:SetScript("OnTextChanged", function(self)
            local text = string.gsub(self:GetText(), "[%s%c%p%a]", "")
            self:SetText(text)
        end)

        itemIDEditBox:SetScript("OnEnterPressed", function(self)
            local itemID = tonumber(self:GetText())
            local isValid = GetItemInfo(itemID)

            if isValid then
                f:SetButtonID(self:GetParent().icon, itemID)
            else
                f:print("Invalid item ID: %s", self:GetText())
            end

            self:ClearFocus()
            self:Hide()
        end)

        itemIDEditBox:Hide()
        button.itemIDEditBox = itemIDEditBox

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Set icon attribute

        if barDB.items[i] then
            f:SetButtonID(button.icon, barDB.items[i].itemID)
        end

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Button visibility

        if i > barDB.numButtons then
            button:Hide()
        else
            button:Show()
        end
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:SetButtonID(icon, itemID, itemTable)
    ClearCursor()

    local button = icon:GetParent()
    local barDB = button:GetParent().db

    button.itemID = itemID

    if itemID then
        if (barDB.items[button.id] and barDB.items[button.id].itemID ~= itemID) or not barDB.items[button.id] then
            barDB.items[button.id] = itemTable or {
                ["itemID"] = itemID,
                ["includeBank"] = false,
                ["objective"] = false,
            }
        end

        db.cache[itemID] = db.cache[itemID] or (GetItemInfo(itemID) and {GetItemInfo(itemID)})
    else
        barDB.items[button.id] = nil
    end

    if not UnitAffectingCombat("player") then
        if itemID then
            icon:SetAttribute("type2", "item")
            icon:SetAttribute("item", "item:" .. itemID)

            icon:RegisterEvent("BAG_UPDATE")
            icon:RegisterEvent("BAG_UPDATE_COOLDOWN")
        else
            icon:SetAttribute("type", nil)
            icon:SetAttribute("item", nil)

            icon:UnregisterEvent("BAG_UPDATE")
            icon:UnregisterEvent("BAG_UPDATE_COOLDOWN")
        end

        local count
        if itemID and (not GetItemInfo(itemID) and db.cache[itemID]) then
            icon:SetNormalTexture(db.cache[itemID][10])
            count = GetItemCount(db.cache[itemID], barDB.items[button.id] and barDB.items[button.id].includeBank)
        elseif itemID and GetItemInfo(itemID) then
            icon:SetNormalTexture(select(10, GetItemInfo(itemID)))
            count = GetItemCount(GetItemInfo(itemID), barDB.items[button.id] and barDB.items[button.id].includeBank)
        elseif itemID then
            local itemInfo = GetItemInfo(itemID)
            count = 0
            C_Timer.After(.1, function()
                icon:SetNormalTexture(select(10, GetItemInfo(itemID)))
                count = GetItemCount(GetItemInfo(itemID), barDB.items[button.id] and barDB.items[button.id].includeBank)
            end)
        else
            icon:SetNormalTexture("")
            count = 0
        end

        icon:GetFontString():SetText(count > 0 and count or "")

        icon.overlay:SetTexture((barDB.items[button.id] and barDB.items[button.id].includeBank and "Interface\\BUTTONS\\UI-AutoCastableOverlay") or "")
        button.objectiveCount:SetText("")
        f:UpdateObjectiveCount(button, barDB.items[button.id] or "")
    else
        icon:RegisterEvent("PLAYER_REGEN_ENABLED")
        f:print(L["Cannot load items during combat. Will resume after combat ends."])
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:UpdateObjectiveCount(button, itemTable)
    if itemTable then
        button.objectiveCount:SetText(itemTable.itemID and f:ParseNumber(itemTable.objective) or "")

        if itemTable and itemTable.objective and GetItemCount(itemTable.itemID, itemTable.includeBank) >= itemTable.objective then
            button.objectiveCount:SetTextColor(0, 1, 0)
        else
            button.objectiveCount:SetTextColor(1, .82, 0)
        end
    else
        button.objectiveCount:SetText("")
    end
end