local addon, ns = ...

local db = setmetatable({}, {__index = function(t, k)
    return _G["FarmingBarDB"][k]
end})
ns.db = db

local profile = setmetatable({}, {__index = function(t, k)
    return _G["FarmingBarCharacterDB"][k]
end})
ns.profile = profile

local L = setmetatable({}, {__index = function(t, k)
    local v = tostring(k)
    rawset(t, k, v)
    return v
end})
ns.L = L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

local f = CreateFrame("Frame", addon .. "Frame", UIParent)
f:SetScript("OnEvent", function(self, event, ...)
    return self[event] and self[event](self, event, ...)
end)

function f:CloseDropDownMenus()
    CloseDropDownMenus()
end
f:SetScript("OnHide", function()
    f:CloseDropDownMenus()
end)
f:Hide()
ns.f = f

tinsert(UISpecialFrames, f:GetName())

f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_REGEN_DISABLED")
f:RegisterEvent("PLAYER_REGEN_ENABLED")

f.LoadBars = {}

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

SLASH_FARMITPLUS1, SLASH_FARMITPLUS2 = "/farmingbar", "/fb"

function SlashCmdList.FARMITPLUS(input)
    local args = {strsplit(' ', input)}
    local cmd = args[1]
    tremove(args, 1)

    if f.ValidCommands[strlower(cmd)] then
        f.ValidCommands[strlower(cmd)](f:unpack(args))
        f:Refresh()
    elseif input ~= "" and not f.ValidCommands[strlower(cmd)] then
        f:print("Invalid command. Available commands are: %s", f:GetValidString(f.ValidCommands))
    else
        f:LoadTab("Commands")
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

f.ChatColors = {
    ["LIGHTRED"] = "|cffff6060",
    ["LIGHTBLUE"] = "|cff00ccff",
    ["TORQUISEBLUE"] = "|cff00C78C",
    ["SPRINGGREEN"] = "|cff00FF7F",
    ["GREENYELLOW"] = "|cffADFF2F",
    ["BLUE"] = "|cff0000ff",
    ["PURPLE"] = "|cffDA70D6",
    ["GREEN"] = "|cff00ff00",
    ["RED"] = "|cffff0000",
    ["GOLD"] = "|cffffcc00",
    ["GOLD2"] = "|cffFFC125",
    ["GREY"] = "|cff888888",
    ["WHITE"] = "|cffffffff",
    ["SUBWHITE"] = "|cffbbbbbb",
    ["MAGENTA"] = "|cffff00ff",
    ["YELLOW"] = "|cffffff00",
    ["ORANGEY"] = "|cffFF4500",
    ["CHOCOLATE"] = "|cffCD661D",
    ["CYAN"] = "|cff00ffff",
    ["IVORY"] = "|cff8B8B83",
    ["LIGHTYELLOW"] = "|cffFFFFE0",
    ["SEXGREEN"] = "|cff71C671",
    ["SEXTEAL"] = "|cff388E8E",
    ["SEXPINK"] = "|cffC67171",
    ["SEXBLUE"] = "|cff00E5EE",
    ["SEXHOTPINK"] = "|cffFF6EB4",
}

f.addonColor = f.ChatColors["SPRINGGREEN"]

local colors = {
    ["%*%a%*"] = f.ChatColors["SPRINGGREEN"], -- addonColor
    ["%*%*%*"] = f.ChatColors["LIGHTBLUE"], --alerts
    ["%*%*"] = f.ChatColors["GREENYELLOW"], -- arguments
    ["%*"] = f.ChatColors["GREEN"], -- commands
}

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:count(tbl)
    local counter = 0
    for k, v in pairs(tbl) do
        counter = counter + 1
    end

    return counter
end

f.pairsByKeys = function(_, t, f)
    local a = {}

    for n in pairs(t) do
        table.insert(a, n)
    end

    table.sort(a, f)

    local i = 0
    local iter = function ()
        i = i + 1
        if a[i] == nil then
            return nil
        else
            return a[i], t[a[i]]
        end
    end

    return iter
end

function f:ParseNumber(number)
    if not number then return end
    number = tonumber(number)

    if number >= 1e9 then
        local num = f:round(number / 1e9, 0)

        if num >= 1000 then
            number = string.format("%dT", f:round(num / 1000, 0))
        else
            number = string.format("%dB", num)
        end
    elseif number >= 1e6 then
        local num = f:round(number / 1e6, 0)

        if num >= 1000 then
            number = string.format("%dB", f:round(num / 1000, 0))
        else
            number = string.format("%dM", num)
        end
    elseif number >= 1000 then
        local num = f:round(number / 1000, 0)

        if num >= 1000 then
            number = string.format("%dM", f:round(num / 1000, 0))
        else
            number = string.format("%dK", num)
        end
    end

    return number
end

function f:ParseText(msg)
    msg = L[msg]

    for k, v in f:pairsByKeys(colors, function(a, b) return a > b end) do
        msg = msg:gsub(string.format("|%s", k), v):gsub(string.format("%s|", k), "|r")
    end

    return msg
end

f.addonString = f:ParseText(string.format("|*a*%s:*a*| ", addon))

function f:print(msg, ...)
    print(f:ParseText(string.format("|*a*%s:*a*| %s", addon, string.format(msg, ...))))
end

function f:round(number, decimals)
    return tonumber((("%%.%df"):format(decimals)):format(number))
end


function f:unpack(tbl)
    if type(tbl) == "table" then
        return unpack(tbl)
    else
        return tbl
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:BuildDatabase()
    FarmingBarDB = FarmingBarDB or {
        templates = {},
        settings = {
            alerts = {
                chat = true,
                screen = true,
                sound = true,
            },
            tooltip = {
                bar = true,
                button = true,
            },
            style = "default",
        },
        cache = {

        }
    }

    FarmingBarCharacterDB = FarmingBarCharacterDB or {bars = {f:NewBar()}}
    FarmingBarCharacterDB.removedBars = FarmingBarCharacterDB.removedBars or {}

    f.namespace = {}
    f.maxButtons = 100
end

function f:GetName()
    return string.format("%s-%s", UnitName("player", false), GetRealmName())
end

function f:GetValidString(tbl, useValue)
    local valid = ""
    for k, v in f:pairsByKeys(tbl) do
        valid = valid == "" and string.format("|**%s**|", useValue and v or k) or string.format("%s | |**%s**|", valid, useValue and v or k)
    end

    return valid
end

function f:ClearCache()
    table.wipe(FarmingBarDB.cache)
    for k, v in pairs(f.bars) do
        if not UnitAffectingCombat("player") then
            f:LoadButtons(v)
        else
            f.LoadBars[v] = true
        end
    end

    StaticPopup_Show("FARMITPLUS_CONFIRM_CLEAR_CACHE")
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:ADDON_LOADED(event, loadedAddon, ...)
    if loadedAddon == addon then
        f:BuildDatabase()
        for k, v in pairs(profile.bars) do
            f:LoadBar(k)
        end
    end
end

function f:PLAYER_REGEN_DISABLED(event, ...)
    f:Refresh()
end

function f:PLAYER_REGEN_ENABLED(event, ...)
    f:Refresh()

    for k, v in pairs(f.LoadBars) do
        f:LoadButtons(k)
    end

    table.wipe(f.LoadBars)
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

StaticPopupDialogs["FARMITPLUS_CONFIRM_APPLY_TEMPLATE"] = {
    text = f.addonString .. f:ParseText("Applying template |**%s**| to bar %d will overwrite buttons. Do you want to continue?"),
    button1 = L["Yes"],
    button2 = L["No"],
    OnAccept = function(_, templateName, barID, templates)
        f:LoadTemplate(templateName, barID, templates)
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}


StaticPopupDialogs["FARMITPLUS_CONFIRM_CLEAR_CACHE"] = {
    text = f.addonString .. f:ParseText("Your item cache has been cleared."),
    button1 = L["Okay"],
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}

StaticPopupDialogs["FARMITPLUS_CONFIRM_DELETE_TEMPLATE"] = {
    text = f.addonString .. f:ParseText("Are you sure you want to delete template |**%s**|?"),
    button1 = L["Yes"],
    button2 = L["No"],
    OnAccept = function(_, templateName)
        f:DeleteTemplate(templateName)
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}

StaticPopupDialogs["FARMITPLUS_CONFIRM_OVERWRITE_TEMPLATE"] = {
    text = f.addonString .. f:ParseText("Template |**%s**| already exists. Do you want to overwrite this template?"),
    button1 = L["Yes"],
    button2 = L["No"],
    OnAccept = function(_, templateName, barID)
        f:SaveTemplate(templateName, barID, true)
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}


StaticPopupDialogs["FARMITPLUS_CONFIRM_RELOAD"] = {
    text = f.addonString .. f:ParseText("Interface reload required to apply new style. Do you want to reload now?"),
    button1 = L["Yes"],
    button2 = L["No"],
    OnAccept = function()
        ReloadUI()
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}

StaticPopupDialogs["FARMITPLUS_CONFIRM_REMOVE_BAR"] = {
    text = f.addonString .. f:ParseText("Removing a bar destroys it completely. Are you sure you want to permanently delete bar %d?"),
    button1 = L["Yes"],
    button2 = L["No"],
    OnAccept = function(_, barID, deleteConfig)
        if deleteConfig then
            f:SetConfigID(nil)
        end
        f:DestroyBar(barID)
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}

StaticPopupDialogs["FARMITPLUS_CONFIRM_REBUILD"] = {
    text = f.addonString .. f:ParseText("This action is irreversible and requires an interface reload. Templates and cache will never be destroyed with this command. Are you sure you want to rebuild: %s settings?"),
    button1 = L["Yes"],
    button2 = L["No"],
    OnAccept = function(_, arg)
        if arg == "profile" then
            FarmingBarCharacterDB = nil
        else
            local tempCache = FarmingBarDB.cache
            local tempTemplates = FarmingBarDB.templates
            FarmingBarDB = nil

            f:BuildDatabase()
            FarmingBarDB.cache = tempCache
            FarmingBarDB.templates = tempTemplates

            if arg == "all" then
                FarmingBarCharacterDB = nil
            end
        end
        ReloadUI()
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}

StaticPopupDialogs["FARMITPLUS_CONFIRM_RESET"] = {
    text = f.addonString .. f:ParseText("This action is irreversible and requires an interface reload. All settings, including cache and templates, will be lost. To specify which settings you want to delete, use the |*rebuild*| command. Do you want to continue?"),
    button1 = L["Yes"],
    button2 = L["No"],
    OnAccept = function()
        FarmingBarCharacterDB = nil
        FarmingBarDB = nil
        ReloadUI()
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}