local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:GetStyle(style)
    local styleTable
    if style == "default" then
        local border = {0, 0, 0, 1}
        local inset = {.05, .05, .05, 1}
        local midRange = {.1, .1, .1, 1}
        local highlight = {1, 1, 1, .5}
        local accent = {1, .82, 0, 1}

        styleTable = {
            bar = {
                type = "texture", 
                texture = "Interface\\BUTTONS\\UI-QuickSlot",
                texCoords = {10/64, 52/64, 10/64, 52/64},
            },

            button = {
                type = "texture", 
                texture = "Interface\\BUTTONS\\UI-EmptySlot",
                texCoords = {10/64, 52/64, 10/64, 52/64},
            },

            buttonEditBox1 = {
                type = "color",
                texture = midRange,
                border = 1,
                borderColor = border,
            },

            buttonEditBox2 = {
                type = "texture", 
                texture = "Interface\\BUTTONS\\UI-EmptySlot",
                texCoords = {10/64, 52/64, 10/64, 52/64},
            },

            fonts = {
                header = "GameFontNormal",
                normal = "GameFontHighlightSmall",
                headerOutline = "GameFontNormalOutline",
                disable = "GameFontDisableSmall",
            },

            elements = {
                checkButton = {
                    size = 32,
                    Normal = {
                        type = "texture",
                        texture = "Interface\\BUTTONS\\UI-CheckBox-Up",
                    },
                    Checked = {
                        type = "texture",
                        texture = "Interface\\AddOns\\FarmingBar\\Textures\\CheckBox-Checked",
                        overlay = true,
                    },
                },

                dropDown = {
                    Normal = {
                        type = "color",
                        texture = midRange,
                        border = 1,
                        borderColor = border,
                    },
                    Highlight = {
                        type = "texture",
                    },
                },

                dropDownMenu = {
                    type = "color",
                    texture = {.1, .1, .1, .9},
                    border = 1,
                    borderColor = border,
                },

                listButton = {
                    size = 12,
                    Normal = {
                        type = "color",
                        texture = midRange,
                        border = 1,
                        borderColor = border,
                    },
                    Checked = {
                        type = "color",
                        texture = highlight,
                        border = 1,
                        borderColor = border,
                    },
                },

                dropDownButton = {
                    Normal = {
                        type = "texture",
                        texture = "Interface\\BUTTONS\\UI-MicroStream-Yellow",
                    },
                },

                dropDownScrollFrame = {
                    upButton = {
                        Normal = {
                            type = "texture",
                            texture = "Interface\\BUTTONS\\UI-MicroStream-Yellow",
                            border = 1,
                            borderColor = border,
                            texCoords = {0, 1, 1, 0},
                            texturePadding = 2,
                        },
                    },
                    downButton = {
                        Normal = {
                            type = "texture",
                            texture = "Interface\\BUTTONS\\UI-MicroStream-Yellow",
                            border = 1,
                            borderColor = border,
                            texCoords = {0, 1, 0, 1},
                            texturePadding = 2,
                        },
                    },
                    thumb = {
                        bgFile = "Interface\\BUTTONS\\WHITE8X8",
                        bgColor = highlight,
                        edgeFile = "Interface\\BUTTONS\\WHITE8X8",
                        edgeSize = 1,
                        size = {12, 22}
                    },
                    track = {
                        bgFile = "Interface\\BUTTONS\\WHITE8X8",
                        bgColor = inset,
                        edgeFile = "Interface\\BUTTONS\\WHITE8X8",
                        edgeSize = 1,
                    },
                },
            },

            optionsFrame = {
                main = {
                    type = "texture", 
                    texture = "Interface\\FrameGeneral\\UI-Background-Rock",
                    border = 10,
                    borderTexture = "Interface\\DialogFrame\\UI-DialogBox-Border",
                },
                inset = {
                    type = "texture",
                    texture = "Interface\\DialogFrame\\UI-DialogBox-Background",
                },
                divider = {
                    bgFile = "Interface\\Common\\ThinBorder-Bottom",
                    height = 16,
                },
                cornerTexture = "Interface\\DialogFrame\\UI-DialogBox-Corner",
            },
        }
    elseif style == "minimal" then
        local border = {0, 0, 0, 1}
        local inset = {.05, .05, .05, 1}
        local midRange = {.1, .1, .1, 1}
        local highlight = {1, 1, 1, .5}
        local button = {0, 0, 0, .75}
        local accent = {1, .82, 0, 1}

        styleTable = {
            bar = {
                type = "color",
                texture = button,
                border = 1,
                borderColor = border,
            },

            button = {
                type = "color",
                texture = button,
                border = 1,
                borderColor = border,
            },

            buttonEditBox1 = {
                type = "color",
                texture = midRange,
                border = 1,
                borderColor = border,
            },

            buttonEditBox2 = {
                type = "color",
                texture = midRange,
                border = 1,
                borderColor = border,
            },

            fonts = {
                header = "GameFontNormal",
                normal = "GameFontHighlightSmall",
                headerOutline = "GameFontNormalOutline",
                disable = "GameFontDisableSmall",
            },

            elements = {
                button = {
                    Normal = {
                        type = "color",
                        texture = midRange,
                        border = 1,
                        borderColor = border,
                        pushedOffset = {0, 0},
                        fonts = {
                            normal = "GameFontHighlightSmall",
                            highlight = "GameFontNormalSmall",
                        },
                    },
                    Highlight = {
                        type = "texture",
                        texture = nil,
                    },
                },

                checkButton = {
                    size = 16,
                    Normal = {
                        type = "color",
                        texture = midRange,
                        border = 1,
                        borderColor = border,
                    },
                    Highlight = {
                        type = "color",
                        texture = highlight,
                        border = 1,
                        borderColor = border,
                    },
                    Checked = {
                        type = "color",
                        texture = accent,
                        border = 1,
                        borderColor = border,
                    },
                },

                dropDown = {
                    Normal = {
                        type = "color",
                        texture = midRange,
                        border = 1,
                        borderColor = border,
                    },
                    Highlight = {
                        type = "texture",
                    },
                },

                dropDownMenu = {
                    type = "color",
                    texture = {.1, .1, .1, .9},
                    border = 1,
                    borderColor = border,
                },

                listButton = {
                    size = 12,
                    Normal = {
                        type = "color",
                        texture = midRange,
                        border = 1,
                        borderColor = border,
                    },
                    Checked = {
                        type = "color",
                        texture = highlight,
                        border = 1,
                        borderColor = border,
                    },
                },

                dropDownButton = {
                    Normal = {
                        type = "texture",
                        texture = "Interface\\BUTTONS\\UI-MicroStream-Yellow",
                    },
                },

                editBox = {
                    type = "color",
                    texture = midRange,
                    border = 1,
                    borderColor = border,
                },

                slider = {
                    type = "color",
                    height = 10,
                    texture = midRange,
                    border = 1,
                    borderColor = border,
                },

                sliderThumb = {
                    type = "color",
                    texture = accent,
                    width = 10,
                },

                scrollFrame = {
                    upButton = {
                        Normal = {
                            type = "texture",
                            texture = "Interface\\BUTTONS\\UI-MicroStream-Yellow",
                            border = 1,
                            borderColor = border,
                            texCoords = {0, 1, 1, 0},
                            texturePadding = 2,
                        },
                    },
                    downButton = {
                        Normal = {
                            type = "texture",
                            texture = "Interface\\BUTTONS\\UI-MicroStream-Yellow",
                            border = 1,
                            borderColor = border,
                            texCoords = {0, 1, 0, 1},
                            texturePadding = 2,
                        },
                    },
                    thumb = {
                        bgFile = "Interface\\BUTTONS\\WHITE8X8",
                        bgColor = highlight,
                        edgeFile = "Interface\\BUTTONS\\WHITE8X8",
                        edgeSize = 1,
                        size = {12, 22}
                    },
                    track = {
                        bgFile = "Interface\\BUTTONS\\WHITE8X8",
                        bgColor = inset,
                        edgeFile = "Interface\\BUTTONS\\WHITE8X8",
                        edgeSize = 1,
                    },
                },
            },

            optionsFrame = {
                main = { 
                    type = "texture",
                    texture = "Interface\\DialogFrame\\UI-DialogBox-Background", 
                    borderTexture = "Interface\\CHARACTERFRAME\\UI-Party-Border",            
                    border = 1,
                },
                inset = { 
                    type = "color",
                    texture = inset,
                    border = 1,
                    borderColor = border,
                },
                divider = {
                    bgFile = "Interface\\CHARACTERFRAME\\UI-Party-Background",
                    height = 1,
                },
                closeButton = {
                    highlight = "GameFontHighlight",
                    normal = "GameFontNormal",
                },
            },
        }
    end

    return styleTable
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

local validButtonTextures = {
    ["Button"] = {"Normal", "Highlight", "Pushed", "Disabled"},
    ["CheckButton"] = {"Normal", "Highlight", "Pushed", "Checked", "DisabledChecked"},
}

function f:SetButtonTexture(button, style, buttonType, desaturateDisabled)
    if not style then
        style = {
            Normal = {
                type = "texture",
                texture = "",
            },
        }
        buttonType = "Button"
    end

    for k, v in f:pairsByKeys(validButtonTextures[buttonType]) do
        local style = style[v] or style["Normal"]

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Setting texture

        if style.type == "color" then
            if button:GetNormalTexture() or button[string.format("Get%sTexture", v)](button) then
                button[string.format("Get%sTexture", v)](button):SetColorTexture(f:unpack(style.texture))
            else
                local texture = button:CreateTexture(nil, "ARTWORK")
                button.texture = texture
                texture:SetAllPoints()
                texture:SetColorTexture(f:unpack(style.texture))
            end
        else
            button[string.format("Set%sTexture", v)](button, style.texture)

            if style.texCoords then
                button[string.format("Get%sTexture", v)](button):SetTexCoord(f:unpack(style.texCoords))
            end
        end

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Transformations

        if button:GetNormalTexture() then
            if style.rotation then
                button[string.format("Get%sTexture", v)](button):SetRotation(style.rotation or 0)
            end

            if style.scale then
                button[string.format("Get%sTexture", v)](button):SetScale(style.scale or 1)
            end

            if style.texCoords then
                button[string.format("Get%sTexture", v)](button):SetTexCoord(f:unpack(style.texCoords))
            end

            if style.texturePadding then
                button[string.format("Get%sTexture", v)](button):SetPoint("TOPLEFT", style.texturePadding, -style.texturePadding)
                button[string.format("Get%sTexture", v)](button):SetPoint("BOTTOMRIGHT", -style.texturePadding, style.texturePadding)
            end

            if not style.noDesaturation and button:GetDisabledTexture() then
                button:GetDisabledTexture():SetDesaturated(true)
            end
        end


        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Border

        local border
        if style.borderColor then
            local padding = (style.texturePadding or 0) - style.border
            border = button:CreateTexture(nil, "BACKGROUND")
            border:SetPoint("TOPLEFT", padding, -padding)
            border:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -padding, padding)
            border:SetColorTexture(f:unpack(style.borderColor))

            button.border = border
            border.size = style.border
        end
    end

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Disabled desaturation

    if button:GetDisabledTexture() and desaturateDisabled then
        button:GetDisabledTexture():SetDesaturated(true)
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:SetFrameTexture(frame, style, clearTexture, overlay)
    if style.type == "color" then
        local colorStyle = { 
            bgFile = "Interface\\BUTTONS\\WHITE8X8",
            edgeFile = style.border and "Interface\\BUTTONS\\WHITE8X8",
            edgeSize = style.border,
        }

        frame:SetBackdrop(colorStyle)
        frame:SetBackdropColor(f:unpack(style.texture))

        if style.borderColor then
            frame:SetBackdropBorderColor(f:unpack(style.borderColor))
        end
    else
        local texture = frame.texture or frame:CreateTexture(nil, "BACKGROUND")
        frame.texture = texture
        texture:SetAllPoints(frame)
        texture:SetTexture(style.texture)


        if style.texCoords then
            texture:SetTexCoord(f:unpack(style.texCoords))
        end

        local border
        if style.borderColor then
            border = frame:CreateTexture(nil, "BORDER")
            border:SetPoint("TOPLEFT", style.border, -style.border)
            border:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -style.border, style.border)

            if style.borderColor then
                border:SetColorTexture(f:unpack(style.borderColor))
            end

            border.size = style.border
            frame.border = border
        end
    end
end