local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

local loaded
function f:LoadCommandsTab(tabID)
    if loaded then return end
    local tabInfo = f.tabs[tabID]
    local scrollContent = f.tabs[tabID].scrollContent

    local header = scrollContent:CreateFontString(nil, "OVERLAY", f:GetStyle(db.settings.style).fonts.header .. "Large")
    header:SetPoint("TOPLEFT", 10, -10)
    header:SetText(tabInfo.title)

    f:RenderElements(scrollContent, header, {
        {
            type = "FontBlock",
            attributes = {
                text =  "This reference documents all available chat commands. Required arguments are noted by {} and optional arguments by (). Commands are accessible through both /farmingbar and /fb.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "|***Chat commands are no longer required. All commands can be executed via the configuration interface, with the exception of controlling multiple bars at once.***|",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*alerts*| {|**chat**||||**screen**||||**sound**|}",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Toggles alerts on or off for the specified channel.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Valid alert channel: |**chat**||||**screen**||||**sound**|",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*alpha*| {|**#**|} (|**#**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Sets the alpha of the specified bar or all bars if none is given.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Alpha value as an integer between 0 and 1.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg2: Valid bar number.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Ex: |***/farmingbar alpha 1 2***| sets the alpha of bar 2 to 1.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*bar*| {|**add**||||**list**||||**remove**|} {#}",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**add**| Adds a new bar with 4 visible buttons and a limit of 12 total buttons. You can add an unlimited number of bars.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**list**| Lists the bar numbers of all available bars.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**remove**| Completely destroys the specified bar and its buttons. Interface reload required.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg2: Valid bar number.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Ex: |***/farmingbar bar remove 1***| removes bar 1.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*grow*| {|**#**|} {|**d**||||**l**||||**r**||||**u**|}",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Controls the direction that a button group grows: (d)own, (l)eft, (r)ight, (u)p.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Valid bar number.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg2: Valid direction: |**d**||||**l**||||**r**||||**u**|.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Ex: |***/farmingbar grow 3 r***| sets bar 3's orientation to horizontal with buttons growing to the right.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*help*|",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Opens the help panel.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*hide*| (|**#**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Hides the specified bar or all bars if none is given.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Valid bar number.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*lock*| (|**#**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Locks the specified bar or all bars if none is given.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Valid bar number.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*rebuild*| (|**all**||||**global**||||**profile**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Rebuilds your specified database. This is an irreversible process that requires an interface reload. Cache and templates will never be erased with this command.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**all**| clears both your global and profile settings.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**profile**| clears only profile settings, such as bars, buttons, and farming objectives.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**global**| clears only global settings, such as alerts, tooltips and styles.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*reset*| (|**all**||||**cache**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Resets the entire database. This is an irreversible process that requires an interface reload. Cache, templates, profiles, and global settings will all be erased with this command.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**all**| resets everything. This argument is optional.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**cache**| clears ONLY your cache and leaves all settings intact. It's recommended you reload your interface after clearing your cache or you may have issues displaying icons the next time you log in. If this happens, a reload will fix the problem.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*scale*| {|**#**|} (|**#**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Scales the specified bar or all bars if none is given.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Scale value as an integer greater than 0.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg2: Valid bar number.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*size*| {|**#**|} (|**#**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Sets the number of visible buttons for the specified bar.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Valid bar number.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = string.format("Arg2: number of visible buttons as an integer between 0 and %d.", f.maxButtons),
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*show*| (|**#**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Shows the specified bar or all bars if none is given.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*style*| (|**default**||||**minimal|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Sets the visual style for bars and options. Requires an interface reload.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**default**| is meant to match the stock WoW interface.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**minimal**| goes better with addons like Bartender or ElvUI.",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*tooltip*| {|**bar**||||**button**|}",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Toggles tooltips on or off for the specified frame.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Valid frame: |**bar**||||**button**|",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*tpl*| {|**delete**||||**list**||||**load**||||**save**|} {|**name**||||**#**|} {|**name**|}",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Describe templates.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**delete**| deletes a user created template by name.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg2: Valid template name, case insensitive.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Ex: |***/farmingbar tpl delete MyTemplate***| deletes the template named \"MyTemplate\".",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**list**| lists all available templates.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**load**| loads a template by name onto the specified bar.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg2: Valid bar number.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg3: Valid template name, case insensitive.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Ex: |***/farmingbar tpl load 1 MyTemplate***| loads the template named \"MyTemplate\" onto bar 1.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: |**save**| saves all items on the specified bar to a template with provided name.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg2: Valid bar number.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg3: Template names are case insensitive and may include special characters and spaces.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Ex: |***/farmingbar tpl save 1 My Template Name!***| saves bar 1 to a farming template named \"My Template Name!\".",
            },
        },
        
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "|*unlock*| (|**#**|)",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Unlocks the specified bar or all bars if none is given.",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Arg1: Valid bar number.",
            },
        },
    }, true)

    loaded = true
end