local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

local loaded
f.configID = nil
function f:LoadConfigTab(tabID)
    if loaded then return end
    local tabInfo = f.tabs[tabID]
    local scrollContent = f.tabs[tabID].scrollContent

    local header = scrollContent:CreateFontString(nil, "OVERLAY",  f:GetStyle(db.settings.style).fonts.header .. "Large")
    header:SetPoint("TOPLEFT", 10, -10)
    header:SetText(tabInfo.title)

    f:RenderElements(scrollContent, header, {
        {
            type = "DropDown",
            attributes = {
                defaultText = "Select Bar",
                justify = "RIGHT",
                onSelect = function(_, selected)
                    f:SetConfigID(selected)
                end,
                onInitialize = function(info)
                    for k, v in f:pairsByKeys(profile.bars) do
                        tinsert(info, {
                            text = string.format("Bar %d", k),
                            arg1 = k,
                            checked = f.configID == k
                        })

                    end
                end,
                onUpdate = function(self)
                    if f:count(profile.bars) == 0 then
                        self:Disable()
                    else
                        self:Enable()
                    end

                    self:SetText(f.configID and string.format(L["Config: %d"], f.configID) or L["Select Bar"])
                end,
            },
            offsetY = -5,
        },
        {
            type = "Button",
            attributes = {
                text = "Add",
                size = {"text", 25},
                func = function()
                    local barID = f:AddBar()
                    f:SetConfigID(barID)
                end,
            },
            group = true,
        },
        {
            type = "Button",
            attributes = {
                text = "Remove",
                size = {"text", 25},
                func = function()
                    if f.configID then
                        local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_REMOVE_BAR", f.configID)
                        if dialog then
                            dialog.data = tonumber(f.configID)
                            dialog.data2 = true
                        end
                    end
                end,
                onUpdate = function(self, ...)
                    if f:count(profile.bars) == 0 or not f.configID or UnitAffectingCombat("player") then
                        self:Disable()
                    else
                        self:Enable()
                    end
                end,
            },
            group = true,
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Controls",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Visible buttons:",
                sizeToText = true,
            },
        },
        {
            type = "EditBox",
            attributes = {
                width = 50,
                validateText = "[^%d]",
                onEnterPressed = function(self, ...)
                    local value = self:GetNumber()
                    if value < 0 then
                        value = 0
                    elseif value > f.maxButtons then
                        value = f.maxButtons
                    end

                    self:SetText(value)
                    f:SetBarSize(f.configID, value)
                end,
                onFocusGained = function(self, ...)
                    if not f.configID then
                        self:ClearFocus()
                    else
                        self:HighlightText()
                    end
                end,
                onUpdate = function(self, ...)
                    if not f.configID or UnitAffectingCombat("player") then
                        self:Disable()
                        self:SetText("")
                        self:ClearFocus()
                    else
                        self:Enable()
                        self:SetText(profile.bars[f.configID].numButtons)
                    end
                end,
                maxLetters = 3,
            },
            group = true,
        },
        {
            type = "CheckButton",
            attributes = {
                text = " ",
                func = function(self)
                    f:TransformBar("movable", not self:GetChecked(), "SetMovable", f.configID)
                    f:print("Bar %d: position %s.", f.configID, self:GetChecked() and "locked" or "unlocked")
                    self:SetText(f.configID and f:ParseText(self:GetChecked() and "locked" or "unlocked") or "")
                end,
                onUpdate = function(self)
                    self:SetEnabled(f.configID)
                    self:SetChecked(f.bars[f.configID] and not f.bars[f.configID]:IsMovable())
                    self:SetText(f.configID and f:ParseText(self:GetChecked() and "locked" or "unlocked") or "")
                end
            },
            group = true,
        },
        {
            type = "CheckButton",
            attributes = {
                text = " ",
                func = function(self)
                    if self:GetChecked() then
                        f:TransformBar("hidden", false, "Show", f.configID)
                    else
                        f:TransformBar("hidden", true, "Hide", f.configID)
                    end
                    self:SetText(f.configID and f:ParseText(self:GetChecked() and "visible" or "hidden") or "")
                end,
                onUpdate = function(self)
                    self:SetEnabled(f.configID and not UnitAffectingCombat("player"))
                    self:SetChecked(f.bars[f.configID] and f.bars[f.configID]:IsVisible())
                    self:SetText(f.configID and f:ParseText(self:GetChecked() and "visible" or "hidden") or "")
                end
            },
            group = true,
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Templates",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Save template as:",
                sizeToText = true,
            },
        },
        {
            type = "EditBox",
            attributes = {
                showButton = true,
                onEnterPressed = function(self, ...)
                    if self:GetText() ~= "" then
                        f:SaveTemplate(self:GetText(), f.configID)
                        self:SetText("")
                    end
                end,
                onFocusGained = function(self, ...)
                    if not f.configID then
                        self:ClearFocus()
                    else
                        self:HighlightText()
                    end
                end,
                onUpdate = function(self, ...)
                    if not f.configID then
                        self:SetText("")
                        self:ClearFocus()
                    end
                end,
                maxLetters = 30,
            },
        },
        {
            type = "DropDown",
            attributes = {
                defaultText = "Load Template",
                justify = "RIGHT",
                onSelect = function(_, selected)
                    local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_APPLY_TEMPLATE", selected, tonumber(f.configID))
                    if dialog then
                        dialog.data = selected
                        dialog.data2 = tonumber(f.configID)
                    end
                end,
                onInitialize = function(info)
                    if f:count(db.templates) > 0 then
                        tinsert(info, {
                            text = "User",
                            isTitle = true,
                        })
                    end
                    for k, v in f:pairsByKeys(db.templates) do
                        tinsert(info, {
                            notCheckable = true,
                            text = k,
                            arg1 = "U:" .. k,
                        })
                    end
                    tinsert(info, {
                        text = "Built-In",
                        isTitle = true,
                    })
                    for k, v in f:pairsByKeys(f.templates) do
                        tinsert(info, {
                            notCheckable = true,
                            text = k,
                            arg1 = "B:" .. k,
                        })
                    end
                end,
                onUpdate = function(self)
                    if not f.configID or UnitAffectingCombat("player") then
                        self:Disable()
                    else
                        self:Enable()
                    end
                end,
            },
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Style",
            },
        },
        {
            type = "DropDown",
            attributes = {
                defaultText = f.configID and string.format(L["Orientation: %s"], profile.bars[f.configID].orientation) or " ",
                justify = "RIGHT",
                onSelect = function(_, selected)
                    profile.bars[f.configID].orientation = selected
                    f:LoadButtons(f.bars[f.configID])
                end,
                onInitialize = function(info)
                    for k, v in f:pairsByKeys(f.ValidArguments["grow"]) do
                        tinsert(info, {
                            text = v,
                            arg1 = v,
                            checked = profile.bars[f.configID].orientation == v
                        })

                    end
                end,
                onUpdate = function(self)
                    if not f.configID or UnitAffectingCombat("player") then
                        self:Disable()
                    else
                        self:Enable()
                    end

                    self:SetText(f.configID and string.format(L["Orientation: %s"], profile.bars[f.configID].orientation) or " ")
                end,
            },
            offsetY = -5,
        },
        {
            type = "Slider",
            attributes = {
                width = 150,
                minValue = 0.25,
                maxValue = 5,
                maxLetters = 4,
                stepValue = .05,
                round = 2,
                orientation = "HORIZONTAL",
                defaultText = "Scale",
                onValueChanged = function(self, value)
                    if f.configID then
                        f:TransformBar("scale", value, "SetScale", f.configID)
                    end
                end,
                onUpdate = function(self)
                    if not f.configID or UnitAffectingCombat("player") then
                        self:Disable()
                    else
                        self:Enable()
                        self:SetValue(profile.bars[f.configID].scale)
                    end
                end,
                onEnterPressed = function(scale, slider)
                    f:TransformBar("scale", scale, "SetScale", f.configID)
                end
            },
            offsetY = -20,
        },
        {
            type = "Slider",
            attributes = {
                width = 150,
                minValue = 0,
                maxValue = 1,
                maxLetters = 4,
                stepValue = .05,
                round = 2,
                orientation = "HORIZONTAL",
                defaultText = "Alpha",
                onValueChanged = function(self, value)
                    if f.configID then
                        f:TransformBar("alpha", value, "SetAlpha", f.configID)
                    end
                end,
                onUpdate = function(self)
                    if not f.configID then
                        self:Disable()
                    else
                        self:Enable()
                        self:SetValue(profile.bars[f.configID].alpha)
                    end
                end,
                onEnterPressed = function(alpha, slider)
                    f:TransformBar("alpha", alpha, "SetAlpha", f.configID)
                end
            },
            offsetX = 20,
            group = true,
        },
    }, true)


    loaded = true
end

function f:Refresh()
    if f:IsVisible() then
        f:Hide()
        f:Show()
    else
        f:Show()
        f:Hide()
    end
end

function f:SetConfigID(barID)
    f.configID = barID
    f:Refresh()
end