local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

local loaded
function f:LoadHelpTab(tabID)
    if loaded then return end
    local tabInfo = f.tabs[tabID]
    local scrollContent = f.tabs[tabID].scrollContent

    local header = scrollContent:CreateFontString(nil, "OVERLAY",  f:GetStyle(db.settings.style).fonts.header .. "Large")
    header:SetPoint("TOPLEFT", 10, -10)
    header:SetText(tabInfo.title)

    f:RenderElements(scrollContent, header, {
        {
            type = "FontBlock",
            attributes = {
                text = "If you need any assistance or have any suggestions/requests, please leave a comment on CurseForge (Twitch) or WoW Interface. You may also message me on Discord |*a*@Niketa#1247*a*| for a faster response, but please limit this to troubleshooting purposes only."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Item Buttons"
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Place any item from your inventory onto a bar slot to keep track of how many you have. Click on an occupied slot to select its contents and move them to another slot. If the destination slot has an item in it already, the items will trade places."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Shift+click a slot to include items from your bank. You don't need to be at the bank for this to work! When enabled, a four-point gold border will appear around the item button. The 'bank included' state is also visible in the button tooltip."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Ctrl+click a slot to set a farming objective for that item. This works similar to WoW's quest tracking. FarmingBar will notify you each time you progress toward your objective and upon reaching your goal. The goal number will turn green once it has been met."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Right-click a slot to use the item."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Shift+right-click a slot to clear it."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Ctrl+right-click any bar slot to manually type in a numeric item ID for an item you want to track."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Bars"
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = string.format("Bars can be switched between vertical and horizontal and can grow in any direction. While each bar has a limit of %d buttons, you can add an unlimited number of bars.", f.maxButtons)
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "To move a bar, click on the bar anchor and drag it to your desired position. You can lock/unlock the bar by shift+clicking or using the lock/unlock chat commands. Bars can be locked all at once or individually when using the chat command."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Right-click the bar anchor to open this help page."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Shift+right-click the bar anchor to configure that bar's settings."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "To grow/shrink the number of visible slots on a bar, click the quick size buttons (-/+). You can also change the number of visible slots with the |*size*| command or in the Config interface."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Bars can be scaled, made transparent, or hidden completely."
            },
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Templates"
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "You can save all items on a bar as a farming template to easily track that group of items again later. Saved templates can be loaded onto any bar. If there is a difference between the amount of items in the template and the size of the bar, the bar will automatically adjust to accommodate the template. Templates do not save item preferences such as farming objectives and bank inclusion. You can use the Config interface or chat commands to load and save templates. Refer to the Commands tab for more information on how to use template chat commands."
            },
        },
    }, true)

    loaded = true
end