local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

local loaded
function f:LoadSettingsTab(tabID)
    if loaded then return end
    local tabInfo = f.tabs[tabID]
    local scrollContent = f.tabs[tabID].scrollContent

    local header = scrollContent:CreateFontString(nil, "OVERLAY",  f:GetStyle(db.settings.style).fonts.header .. "Large")
    header:SetPoint("TOPLEFT", 10, -10)
    header:SetText(tabInfo.title)

    f:RenderElements(scrollContent, header, {
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Reset Settings",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Global settings: alerts, tooltips, styles",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Profile settings: bars",
            },
        },
        {
            type = "FontBlock",
            attributes = {
                text = "Cache: stores item info to load icon textures on login.",
            },
        },
        {
            type = "Button",
            attributes = {
                text = "Rebuild Global",
                size = {"text", 25},
                func = function()
                    local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_REBUILD", "global")
                    if dialog then
                        dialog.data = "global"
                    end
                end,
            },
            offsetY = -5,
        },
        {
            type = "Button",
            attributes = {
                text = "Rebuild Profile",
                size = {"text", 25},
                func = function()
                    local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_REBUILD", "profile")
                    if dialog then
                        dialog.data = "profile"
                    end
                end,
            },
            group = true,
        },
        {
            type = "Button",
            attributes = {
                text = "Clear Cache",
                size = {"text", 25},
                func = function()
                    f:ClearCache()
                end
            },
            group = true,
        },
        {
            type = "Button",
            attributes = {
                text = "Reset All",
                size = {"text", 25},
                func = function()
                    local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_RESET")
                    if dialog then
                        dialog.data = "all"
                    end
                end
            },
            group = true,
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Objective Alerts",
            },
        },
        {
            type = "CheckButton",
            attributes = {
                text = "Chat alerts",
                func = function(self)
                    db.settings.alerts.chat = self:GetChecked()
                end,
                onUpdate = function(self)
                    self:SetChecked(db.settings.alerts.chat)
                end,
            },
        },
        {
            type = "CheckButton",
            attributes = {
                text = "Screen alerts",
                func = function(self)
                    db.settings.alerts.screen = self:GetChecked()
                end,
                onUpdate = function(self)
                    self:SetChecked(db.settings.alerts.screen)
                end,
            },
            group = true,
        },
        {
            type = "CheckButton",
            attributes = {
                text = "Sound alerts",
                func = function(self)
                    db.settings.alerts.sound = self:GetChecked()
                end,
                onUpdate = function(self)
                    self:SetChecked(db.settings.alerts.sound)
                end,
            },
            group = true,
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Tooltips",
            },
        },
        {
            type = "CheckButton",
            attributes = {
                text = "Bar tooltips",
                func = function(self)
                    db.settings.tooltip.bar = self:GetChecked()
                end,
                onUpdate = function(self)
                    self:SetChecked(db.settings.tooltip.bar)
                end,
            },
        },
        {
            type = "CheckButton",
            attributes = {
                text = "Button tooltips",
                func = function(self)
                    db.settings.tooltip.button = self:GetChecked()
                end,
                onUpdate = function(self)
                    self:SetChecked(db.settings.tooltip.button)
                end,
            },
            group = true,
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Styles",
            },
        },
        {
            type = "DropDown",
            attributes = {
                defaultText = string.format("Style: %s", db.settings.style),
                justify = "RIGHT",
                onSelect = function(self, selected)
                    self:SetText(string.format("Style: %s", selected))
                    db.settings.style = selected
                    StaticPopup_Show("FARMITPLUS_CONFIRM_RELOAD")
                end,
                onInitialize = function(info)
                    for k, v in f:pairsByKeys(f.ValidArguments["style"]) do
                        tinsert(info, {
                            text = v,
                            arg1 = v,
                            checked = db.settings.style == v
                        })

                    end
                end,
                onUpdate = function(self)
                    self:SetText(string.format("Style: %s", db.settings.style))
                end,
            },
            offsetY = -5,
        },
        {
            type = "FontBlock",
            attributes = {
                header = true,
                text = "Manage Templates",
            },
        },
        {
            type = "DropDown",
            attributes = {
                defaultText = "Delete Template",
                justify = "RIGHT",
                onSelect = function(_, selected)
                    local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_DELETE_TEMPLATE", selected)
                    if dialog then
                        dialog.data = selected
                    end
                end,
                onInitialize = function(info)
                    for k, v in f:pairsByKeys(db.templates) do
                        tinsert(info, {
                            notCheckable = true,
                            text = k,
                            arg1 = k,
                        })

                    end
                end,
                onUpdate = function(self)
                    if f:count(db.templates) == 0 then
                        self:Disable()
                    else
                        self:Enable()
                    end
                end,
            },
            offsetY = -5,
        },
    }, true)

    loaded = true
end
